@@ -1,7 +1,7 @@
 /*
  * FrodoKEM matrix logic
  * Based on the MIT licensed reference implementation by the designers
- * (https://github.com/microsoft/PQCrypto-LWEKE/tree/master/src)
+ * (https://github.com/microsoft/PQCrypto-LWEKE/tree/master)
  *
  * The Fellowship of the FrodoKEM:
  * (C) 2023 Jack Lloyd
@@ -24,7 +24,7 @@
 
 namespace Botan {
 
-class FrodoMatrix {
+class FrodoMatrix final {
    public:
       using Dimensions = std::tuple<size_t, size_t>;
 
