@@ -7,11 +7,12 @@
 #include "tests.h"
 
 #include <botan/hex.h>
-#include <botan/internal/cpuid.h>
 #include <botan/internal/filesystem.h>
 #include <botan/internal/fmt.h>
+#include <botan/internal/loadstor.h>
 #include <botan/internal/parsing.h>
 #include <botan/internal/stl_util.h>
+#include <botan/internal/target_info.h>
 #include <fstream>
 #include <iomanip>
 #include <sstream>
@@ -20,7 +21,11 @@
    #include <botan/bigint.h>
 #endif
 
-#if defined(BOTAN_HAS_EC_CURVE_GFP)
+#if defined(BOTAN_HAS_CPUID)
+   #include <botan/internal/cpuid.h>
+#endif
+
+#if defined(BOTAN_HAS_LEGACY_EC_POINT)
    #include <botan/ec_point.h>
 #endif
 
@@ -75,7 +80,7 @@ void Test::Result::test_note(const std::string& note, const char* extra) {
    if(!note.empty()) {
       std::ostringstream out;
       out << who() << " " << note;
-      if(extra) {
+      if(extra != nullptr) {
          out << ": " << extra;
       }
       m_log.push_back(out.str());
@@ -103,7 +108,7 @@ bool Test::Result::ThrowExpectations::check(const std::string& test_name, Test::
       if(m_expect_success) {
          return result.test_failure(test_name + " threw unexpected exception: " + ex.what());
       }
-      if(m_expected_exception_type.has_value() && m_expected_exception_type.value() != typeid(ex)) {
+      if(m_expected_exception_check_fn && !m_expected_exception_check_fn(std::current_exception())) {
          return result.test_failure(test_name + " threw unexpected exception: " + ex.what());
       }
       if(m_expected_message.has_value() && m_expected_message.value() != ex.what()) {
@@ -111,7 +116,7 @@ bool Test::Result::ThrowExpectations::check(const std::string& test_name, Test::
                                     m_expected_message.value() + "', got: '" + ex.what() + "')");
       }
    } catch(...) {
-      if(m_expect_success || m_expected_exception_type.has_value() || m_expected_message.has_value()) {
+      if(m_expect_success || m_expected_exception_check_fn || m_expected_message.has_value()) {
          return result.test_failure(test_name + " threw unexpected unknown exception");
       }
    }
@@ -190,7 +195,7 @@ bool Test::Result::test_eq(const char* producer,
 
    err << who();
 
-   if(producer) {
+   if(producer != nullptr) {
       err << " producer '" << producer << "'";
    }
 
@@ -205,7 +210,9 @@ bool Test::Result::test_eq(const char* producer,
 
    for(size_t i = 0; i != xor_diff.size(); ++i) {
       xor_diff[i] = produced[i] ^ expected[i];
-      bytes_different += (xor_diff[i] > 0);
+      if(xor_diff[i] > 0) {
+         bytes_different++;
+      }
    }
 
    err << "\nProduced: " << Botan::hex_encode(produced, produced_size)
@@ -335,7 +342,7 @@ bool Test::Result::test_ne(const std::string& what, const BigInt& produced, cons
 }
 #endif
 
-#if defined(BOTAN_HAS_EC_CURVE_GFP)
+#if defined(BOTAN_HAS_LEGACY_EC_POINT)
 bool Test::Result::test_eq(const std::string& what, const Botan::EC_Point& a, const Botan::EC_Point& b) {
    //return test_is_eq(what, a, b);
    if(a == b) {
@@ -391,7 +398,7 @@ bool Test::Result::test_rc(const std::string& func, int expected, int rc) {
 
 void Test::initialize(std::string test_name, CodeLocation location) {
    m_test_name = std::move(test_name);
-   m_registration_location = std::move(location);
+   m_registration_location = location;
 }
 
 Botan::RandomNumberGenerator& Test::rng() const {
@@ -402,18 +409,18 @@ Botan::RandomNumberGenerator& Test::rng() const {
    return *m_test_rng;
 }
 
-std::vector<std::string> Test::possible_providers(const std::string& /*unused*/) {
+std::vector<std::string> Test::possible_providers(const std::string& /*alg*/) {
    return Test::provider_filter({"base"});
 }
 
 //static
-std::string Test::format_time(uint64_t ns) {
+std::string Test::format_time(uint64_t nanoseconds) {
    std::ostringstream o;
 
-   if(ns > 1000000000) {
-      o << std::setprecision(2) << std::fixed << ns / 1000000000.0 << " sec";
+   if(nanoseconds > 1000000000) {
+      o << std::setprecision(2) << std::fixed << nanoseconds / 1000000000.0 << " sec";
    } else {
-      o << std::setprecision(2) << std::fixed << ns / 1000000.0 << " msec";
+      o << std::setprecision(2) << std::fixed << nanoseconds / 1000000.0 << " msec";
    }
 
    return o.str();
@@ -448,7 +455,7 @@ std::string Test::Result::result_string() const {
       report << " in " << format_time(m_ns_taken);
    }
 
-   if(tests_failed()) {
+   if(tests_failed() > 0) {
       report << " " << tests_failed() << " FAILED";
    } else {
       report << " all ok";
@@ -541,7 +548,7 @@ class Test_Registry {
 
          // TODO: this is O(n^2), but we have a relatively small number of tests.
          auto insert_if_not_exists_and_not_skipped = [&](const std::string& test_name) {
-            if(!Botan::value_exists(result, test_name) && to_be_skipped.find(test_name) == to_be_skipped.end()) {
+            if(!Botan::value_exists(result, test_name) && !to_be_skipped.contains(test_name)) {
                result.push_back(test_name);
             }
          };
@@ -558,7 +565,7 @@ class Test_Registry {
             }
          } else {
             for(const auto& r : requested) {
-               if(m_tests.find(r) != m_tests.end()) {
+               if(m_tests.contains(r)) {
                   insert_if_not_exists_and_not_skipped(r);
                } else if(auto elems = m_categories.equal_range(r); elems.first != m_categories.end()) {
                   for(; elems.first != elems.second; ++elems.first) {
@@ -652,7 +659,7 @@ std::string Test::temp_file_name(const std::string& basename) {
    // POSIX only calls for 6 'X' chars but OpenBSD allows arbitrary amount
    std::string mkstemp_basename = "/tmp/" + basename + ".XXXXXXXXXX";
 
-   int fd = ::mkstemp(&mkstemp_basename[0]);
+   int fd = ::mkstemp(mkstemp_basename.data());
 
    // error
    if(fd < 0) {
@@ -752,9 +759,7 @@ class Testsuite_RNG final : public Botan::RandomNumberGenerator {
          }
       }
 
-      Testsuite_RNG(std::string_view seed, std::string_view test_name) {
-         m_x = 0;
-
+      Testsuite_RNG(std::string_view seed, std::string_view test_name) : m_x(0) {
          for(char c : seed) {
             this->mix(static_cast<uint8_t>(c));
          }
@@ -825,11 +830,11 @@ std::string Test::data_file_as_temporary_copy(const std::string& what) {
 }
 
 //static
-std::vector<std::string> Test::provider_filter(const std::vector<std::string>& in) {
+std::vector<std::string> Test::provider_filter(const std::vector<std::string>& providers) {
    if(m_opts.provider().empty()) {
-      return in;
+      return providers;
    }
-   for(auto&& provider : in) {
+   for(auto&& provider : providers) {
       if(provider == m_opts.provider()) {
          return std::vector<std::string>{provider};
       }
@@ -842,6 +847,10 @@ std::string Test::random_password(Botan::RandomNumberGenerator& rng) {
    return Botan::hex_encode(rng.random_vec(len));
 }
 
+size_t Test::random_index(Botan::RandomNumberGenerator& rng, size_t max) {
+   return Botan::load_be(rng.random_array<8>()) % max;
+}
+
 std::vector<std::vector<uint8_t>> VarMap::get_req_bin_list(const std::string& key) const {
    auto i = m_vars.find(key);
    if(i == m_vars.end()) {
@@ -1000,9 +1009,7 @@ Botan::BigInt VarMap::get_req_bn(const std::string& key) const {
    }
 }
 
-Botan::BigInt VarMap::get_opt_bn(const std::string& key, const Botan::BigInt& def_value) const
-
-{
+Botan::BigInt VarMap::get_opt_bn(const std::string& key, const Botan::BigInt& def_value) const {
    auto i = m_vars.find(key);
    if(i == m_vars.end()) {
       return def_value;
@@ -1056,11 +1063,13 @@ std::string Text_Based_Test::get_next_line() {
          m_cur = std::make_unique<std::ifstream>(m_srcs[0]);
          m_cur_src_name = m_srcs[0];
 
+#if defined(BOTAN_HAS_CPUID)
          // Reinit cpuid on new file if needed
          if(m_cpu_flags.empty() == false) {
             m_cpu_flags.clear();
             Botan::CPUID::initialize();
          }
+#endif
 
          if(!m_cur->good()) {
             throw Test_Error("Could not open input file '" + m_cur_src_name);
@@ -1078,7 +1087,7 @@ std::string Text_Based_Test::get_next_line() {
          }
 
          if(line[0] == '#') {
-            if(line.compare(0, 6, "#test ") == 0) {
+            if(line.starts_with("#test ")) {
                return line;
             } else {
                continue;
@@ -1106,12 +1115,18 @@ std::string strip_ws(const std::string& in) {
    return in.substr(first_c, last_c - first_c + 1);
 }
 
-std::vector<uint64_t> parse_cpuid_bits(const std::vector<std::string>& tok) {
-   std::vector<uint64_t> bits;
+std::vector<std::string> parse_cpuid_bits(const std::vector<std::string>& tok) {
+   std::vector<std::string> bits;
+
+#if defined(BOTAN_HAS_CPUID)
    for(size_t i = 1; i < tok.size(); ++i) {
-      const std::vector<Botan::CPUID::CPUID_bits> more = Botan::CPUID::bit_from_string(tok[i]);
-      bits.insert(bits.end(), more.begin(), more.end());
+      if(auto bit = Botan::CPUID::bit_from_string(tok[i])) {
+         bits.push_back(bit->to_string());
+      }
    }
+#else
+   BOTAN_UNUSED(tok);
+#endif
 
    return bits;
 }
@@ -1125,7 +1140,8 @@ bool Text_Based_Test::skip_this_test(const std::string& /*header*/, const VarMap
 std::vector<Test::Result> Text_Based_Test::run() {
    std::vector<Test::Result> results;
 
-   std::string header, header_or_name = m_data_src;
+   std::string header;
+   std::string header_or_name = m_data_src;
    VarMap vars;
    size_t test_cnt = 0;
 
@@ -1136,7 +1152,7 @@ std::vector<Test::Result> Text_Based_Test::run() {
          break;
       }
 
-      if(line.compare(0, 6, "#test ") == 0) {
+      if(line.starts_with("#test ")) {
          std::vector<std::string> pragma_tokens = Botan::split_on(line.substr(6), ' ');
 
          if(pragma_tokens.empty()) {
@@ -1187,7 +1203,7 @@ std::vector<Test::Result> Text_Based_Test::run() {
 
       if(key == m_output_key) {
          try {
-            for(auto& req_key : m_required_keys) {
+            for(const auto& req_key : m_required_keys) {
                if(!vars.has_key(req_key)) {
                   auto r =
                      Test::Result::Failure(header_or_name, Botan::fmt("{} missing required key {}", test_id, req_key));
@@ -1204,20 +1220,23 @@ std::vector<Test::Result> Text_Based_Test::run() {
             uint64_t start = Test::timestamp();
 
             Test::Result result = run_one_test(header, vars);
+#if defined(BOTAN_HAS_CPUID)
             if(!m_cpu_flags.empty()) {
-               for(const auto& cpuid_u64 : m_cpu_flags) {
-                  Botan::CPUID::CPUID_bits cpuid_bit = static_cast<Botan::CPUID::CPUID_bits>(cpuid_u64);
-                  if(Botan::CPUID::has_cpuid_bit(cpuid_bit)) {
-                     Botan::CPUID::clear_cpuid_bit(cpuid_bit);
-                     // now re-run the test
-                     result.merge(run_one_test(header, vars));
+               for(const auto& cpuid_str : m_cpu_flags) {
+                  if(const auto bit = Botan::CPUID::Feature::from_string(cpuid_str)) {
+                     if(Botan::CPUID::has(*bit)) {
+                        Botan::CPUID::clear_cpuid_bit(*bit);
+                        // now re-run the test
+                        result.merge(run_one_test(header, vars));
+                     }
                   }
                }
                Botan::CPUID::initialize();
             }
+#endif
             result.set_ns_consumed(Test::timestamp() - start);
 
-            if(result.tests_failed()) {
+            if(result.tests_failed() > 0) {
                std::ostringstream oss;
                oss << "Test # " << test_cnt << " ";
                if(!header.empty()) {
